import { TableCellBaseProps, TableRowProps } from '@material-ui/core';
import * as React from 'react';

export declare interface ChildrenFunction {
  /**
   * Função sera executada quando for clicado em alguma linha, seu retorno aparecerá abaixo da linha clicada
   * 
   * @param linhaOriginal - Linha original, antes de ser formatada pela função line
   * @param linha - Linha ja formatada pela função line
   * @param index - Índice da linha
   * 
   * @returns {React.ReactNode} Componente react 
   */ 
  (props: {linhaOriginal: Array<any> | object, linha: Array<any>, index:number}): React.ReactNode
}

export declare interface ColumnOptions {
  /**
   * Indice da coluna a ser modificada
   */ 
  alias: number,
  /**
   * JSX Customizado para a coluna
   * 
   * @param itemFormatado - Item retornado pela funcao line no indice selecionado
   * @param linhaOriginal - Linha original, antes de ser formatada pela função line
   * @param linhaFormatada - Linha ja formatada pela função line
   * @param index - Índice da linha
   * 
   * @returns {React.ReactNode} Componente react 
   */ 
  customJsx: (props: {index: number, itemFormatado: any, linhaFormatada: Array<any>, linhaOriginal: Array<any> | object}) => React.ReactNode
}

export interface ArgousComplexTableBodyProps {
  /**
   * @ignore
   */
  children?: React.ReactNode | ChildrenFunction,
  /**
   * Função que converte a linha original em um array com os dados a serem exibidos
   * 
   * @param linhaOriginal - linha a ser formatada
   * 
   * @returns {Array<any>} Retorna array
   */ 
  line: (linhaOriginal: Array<Array<any> | Object>) => Array<any>, 
  /**
   * Estilo que pode ser aplicado por linhas dependendo do conteúdo ou do índice da linha 
   * 
   * @param linhaOriginal - Linha original
   * @param linhaFormatada - Linha formatada
   * @param index - Índice da linha
   * 
   * @returns {React.CSSProperties} Estilos para a linha
   */ 
  lineStyle?: (linhaOriginal: Array<any> | object, linhaFormatada: Array<any>, index:number) => React.CSSProperties,
  /**
   * Classe que pode ser aplicada por linhas dependendo do conteúdo ou do índice da linha 
   * 
   * @param linhaFormatada - Linha formatada
   * @param linhaOriginal - Linha original
   * @param index - Índice da linha
   * 
   * @returns {string} Classe para a linha
   */ 
  lineClass?: (linhaFormatada: Array<any>, linhaOriginal: Array<any> | object, index: number) => string,
  /**
   * Habilita a seleção de linhas da lista 
   */ 
  enableCheck?: boolean,
  /**
   * Habilita o drag and drop das linhas
   */ 
  enableDrag?: boolean,
  /**
   * Função chamada ao clicar em uma linha
   * 
   * @param evento - Evento do drop
   * 
   * @returns {void} Sem retorno
   */ 
  onDrop?: (evento: React.DragEventHandler<HTMLDivElement>) => void,
  /**
   * Função que decide se a seleção de determinada linha está habilitada
   * 
   * @param linhaOriginal - Linha original, antes de ser formatada pela função line
   * @param linhaFormatada - Linha ja formatada pela função line
   * @param index - Índice da linha
   * 
   * @returns {boolean} Booleano
   */ 
  disableCheckIf?: (linhaOriginal: Array<any> | object, linhaFormatada: Array<any>, index: number) => boolean,
  /**
   * Array de estilos para cada coluna da linha
   */ 
  columnStyles?: Array<React.CSSProperties>,
  /**
   * Função chamada ao clicar em uma linha
   * 
   * @param evento - Evento do click
   * @param linhaOriginal - Linha original
   * @param linhaFormatada - Linha formatada
   * @param index - Índice da linha clicada
   * 
   * @returns {void} Sem retorno
   */ 
  onClick?: (evento: EventTarget, linhaOriginal: Array<any> | object, linhaFormatada: Array<any>, index:number) => void,
  /**
   * Ìndice da linha original a ser utilizado como ID primário na hora das seleções e que será retornado pelas actions
   */ 
  indexToSelect?: number | string,
  /**
   * Array de opções para as colunas
   * 
   * @param alias - Índice da coluna
   * @param customJsx - Função que retorna o JSX customizado
   */ 
  columnOptions?: Array<ColumnOptions>,
  /**
   * Title que aparecerá ao dar hover na linha
   */ 
  lineTitle?: string,
  /**
   * Item da linha original a ser utilizado como key no map
   */ 
  customKey?: string | number,
  /**
   * Props a serem repassados para a celula da tabela
   * 
   * @param index - Index da coluna sendo testada
   * @param coluna - Item da coluna sendo testada
   * @param linhaOriginal - Linha original sendo testada
   * @param linhaFormatada - Linha formatada sendo testada
   * 
   * @returns {TableCellBaseProps} Props a serem passados para a coluna
   */ 
  columnProps?: (index: number, coluna: any, linhaOriginal: Array<any> | object, linhaFormatada: Array<any>) => TableCellBaseProps,
  /**
   * Props a serem repassados para a linha da tabela
   * 
   * @param index - Index da coluna sendo testada
   * @param linhaOriginal - Linha original sendo testada
   * @param linhaFormatada - Linha formatada sendo testada
   * @param itensSelecionados - Linha formatada sendo testada
   * 
   * @returns {TableRowProps} Props a serem passados para a linha
   */ 
  rowProps?: (props: {index: number, linhaOriginal: Array<any> | object, linhaFormatada: Array<any>, itensSelecionados: any[]}) => TableRowProps
}

export interface ArgousComplexTableBodyPrivateProps extends ArgousComplexTableBodyProps {
  /**
   * Função que abre a linha clicada
   * 
   * @param index - Índice da linha clicada
   * 
   * @returns {void} Sem retorno
   */
  handleOpenClose: (index: number) => void,
  /**
   * Array de títulos para o cabeçalho da tabela
   */ 
  head?: Array<string>, 
  /**
   * Índice aberto 
   */
  open: number,
  /**
   * Array de dados que estão visíveis no momento
   */
  dadosMostrados: Array<Array<any> | Object>,
}

declare function ArgousComplexTableBody(props: ArgousComplexTableBodyPrivateProps): JSX.Element

export default ArgousComplexTableBody;