import { dataUrlToBlob } from '../../utils';
import './cam.css';
import moment from 'moment';
import React, { useCallback, useEffect, useRef } from 'react';


const GET_STREAM_INTERVAL_TIMEOUT = 3000;

const VIDEO_ID = 'cam-video';
const CANVAS_ID = 'cam-canvas';

const VIDEO_CONSTRAINTS = { 
  audio: false, 
  video: { 
    facingMode: 'user' 
  } 
};

const createVideoElementId = (id = '') => (id) ? `${id}-${VIDEO_ID}` : VIDEO_ID;
const createCanvasElementId = (id = '') => (id) ? `${id}-${CANVAS_ID}` : CANVAS_ID;

export const camSnapshot = ({ e = undefined, download = false, camComponentId = '' }) => {
  let status = false, file = undefined, linkElement = undefined, canvasElement = undefined;
  const videoElementId = createVideoElementId(camComponentId);
  const videoElement = document.getElementById(videoElementId);

  if (videoElement) {
    canvasElement = document.createElement('canvas');
    if (canvasElement) {
      const canvasElementId = createCanvasElementId(camComponentId);
      canvasElement.id = canvasElementId;
      const canvasContext2D = canvasElement.getContext('2d');
      if (canvasContext2D) {
        canvasElement.width = videoElement.offsetWidth;
        canvasElement.height = videoElement.offsetHeight;
        canvasContext2D.drawImage(videoElement, 0, 0, videoElement.offsetWidth, videoElement.offsetHeight);

        linkElement = document.createElement('a');
        linkElement.download = `Capture_${moment(new Date()).format('DD-MM-YYYY_HH:mm:ss')}.png`;
        const dataUrl = canvasElement.toDataURL('image/png');
        linkElement.href = dataUrl;

        const blob = dataUrlToBlob(dataUrl);
        if (blob) file = new File([blob], linkElement.download, { type: blob.type });
      }
    }
  }
  (linkElement && file) && (status = true);
  (linkElement && download) && linkElement.click();
  return { e, status, file, link: linkElement, canvas: canvasElement };
};


const Cam = ({ id = '', style = {}, onGetStream }) => {

  const videoRef = useRef(null);
  const streamRef = useRef(null);
  const getStreamIntervalIdRef = useRef(null);
  const videoElementId = createVideoElementId(id);

  const getStream = useCallback(async () => {
    let stream = null;

    try {
      stream = await navigator.mediaDevices.getUserMedia(VIDEO_CONSTRAINTS);
      stream.getTracks().forEach((track) => {
        if (track.kind === 'video') track.enabled = true;
      });
    } catch (error) {
      console.error(error);
    }

    if ((stream && !streamRef.current) || (!stream && streamRef.current)) {
      videoRef.current.srcObject = stream;
      stopVideo(streamRef.current);
      streamRef.current = stream;
      (typeof onGetStream === 'function') && onGetStream({ stream });
    } else if (stream) stopVideo(stream);
  }, [onGetStream]);

  const stopVideo = (stream) => (stream) && stream.getTracks().forEach((track) => {
    track?.stop();
    track.enabled = false;
  });

  useEffect(() => {
    getStream();
    getStreamIntervalIdRef.current = setInterval(() => getStream(), GET_STREAM_INTERVAL_TIMEOUT);
    return () => {
      clearInterval(getStreamIntervalIdRef.current);
      stopVideo(streamRef.current);
    };
  }, [getStream]);


  return <div style={style} className='cam-div'>
    <video id={videoElementId} ref={videoRef} autoPlay />
  </div>;
};


export default Cam;