import { makeStyles } from '@material-ui/core';
import clsx from 'clsx';
import { isNumber } from 'lodash';
import React, { useCallback, useEffect, useMemo } from 'react';

const ELLIPSIS_TEXT_STYLE = {
  overflow: 'hidden',
  whiteSpace: 'nowrap',
  textOverflow: 'ellipsis'
};

const useStyles = makeStyles({
  container: {
    display: 'flex',
    height: ({ size }) => `${size / 2}px`,
    transition: 'all .3s',
    opacity: ({ responsive }) => responsive ? 0 : 1
  },
  gauge: {
    height: ({ size }) => size,
    width: ({ size }) => size,
    backgroundColor: ({ barBackgroundColor }) => barBackgroundColor,
    borderRadius: '50%',
    display: 'grid',
    placeItems: 'center',
    webkitClipPath: 'polygon(0 0, 100% 0, 100% 51%, 0 51%)',
    clipPath: 'polygon(0 0, 100% 0, 100% 51%, 0 51%)',
    '&::before': {
      content: '""',
      position: 'absolute',
      height: ({ gaugeBarWidth }) => gaugeBarWidth,
      width: ({ gaugeBarWidth }) => gaugeBarWidth,
      borderRadius: '50%',
      backgroundColor: ({ backgroundColor }) => backgroundColor
    }
  },
  valueDiv: {
    position: 'absolute',
    color: ({ color }) => color,
    fontSize: ({ fontSize }) => fontSize,
    maxWidth: ({ size }) => `${size / 2}px`,
    minWidth: ({ size }) => `${size / 2}px`,
    maxHeight: ({ size }) => `${size / 4}px`,
    minHeight: ({ size }) => `${size / 4}px`,
    transform: ({ size }) => `translateY(-${(size / 4) / 2}px)`,
    display: 'flex',
    flexDirection: 'column',
    alignItems: 'center',
    justifyContent: 'end',
    overflow: 'hidden',
    whiteSpace: 'nowrap',
    padding: 4,
    '&>p': {
      ...ELLIPSIS_TEXT_STYLE,
      margin: 0,
      width: '100%',
      textAlign: 'center',
      '&:first-child': {
        fontWeight: 'bold'
      }
    }
  },
  minMax: {
    display: 'flex',
    alignItems: 'end',
    padding: '0 1rem',
    width: ({ minMaxWidth }) => minMaxWidth,
    color: ({ color }) => color,
    fontSize: ({ fontSize }) => fontSize,
    '&>p': {
      margin: 0
    }
  },
  min: {
    justifyContent: 'end'
  }
});


const getHand = (percentage, handColor) => {
  if (!isNumber(percentage)) return '';
  
  let angle = 270;

  if (percentage > 50) {
    angle = 0;
    percentage -= 50;
  }

  const initialTransparentAngle = angle;

  angle = angle + (percentage * 1.8);

  let finalTransparentAngle = angle + 2;

  if (finalTransparentAngle > 90 && finalTransparentAngle < 270) {
    finalTransparentAngle = 90;
  }
  
  return `conic-gradient(
    #0000 ${initialTransparentAngle}deg ${angle - 2}deg,
    ${handColor} ${angle - 0.5}deg ${angle + 0.5}deg,
    #0000 ${finalTransparentAngle}deg
  )`;
};


const HalfGauge = ({ 
  responsive = false,
  id = '',
  label = '',
  percentage = 0,
  order = 'D',
  min = '',
  max = '',
  size = 200,
  fontSize = '0.9rem',
  color = '#000',
  backgroundColor = '#fff',
  barBackgroundColor = '#0000',
  handColor = '#fff',
  minMaxWidth = '6rem',
  middleColor = '#ffdd71',
  middleRightColor = '#fec900',
  rightColor = '#00995d',
  leftColor = '#fe6c6c',
  middleLeftColor = '#ff9b9b'
}) => {

  const gaugeBarWidth = size - (size / 2);
  const classes = useStyles({ color, backgroundColor, barBackgroundColor, size, gaugeBarWidth, fontSize, minMaxWidth, responsive });
  
  const halfGaugeId = `${id}-halfGauge`;
  const halfGaugeHandId = `${id}-halfGaugeHand`;
  const halfGaugeContainerId = `${id}-halfGaugeContainer`;
  const percentageText = `${percentage.toString().replace('.', ',')}%`;

  const defaultColors = useMemo(() => (
    [middleColor, middleRightColor, rightColor, '#0000', leftColor, middleLeftColor]
  ), [leftColor, middleColor, middleLeftColor, middleRightColor, rightColor]);

  const gaugeBackgroundColors = useMemo(() => {
    let colors = defaultColors;

    if (order === 'A') {
      const left = defaultColors[4];
      const middleLeft = defaultColors[5];
      const middleRight = defaultColors[1];
      const right = defaultColors[2];

      colors.splice(4, 1, right);
      colors.splice(5, 1, middleRight);
      colors.splice(1, 1, middleLeft);
      colors.splice(2, 1, left);
    }

    return colors;
  }, [defaultColors, order]);


  const createStage = useCallback(() => {
    const gaugeEl = document.getElementById(halfGaugeId);
    if (!gaugeEl) return;

    gaugeEl.style.background = `conic-gradient(
      ${gaugeBackgroundColors[0]} 18deg, 
      ${gaugeBackgroundColors[1]} 18deg 54deg,
      ${gaugeBackgroundColors[2]} 54deg 90deg,
      ${gaugeBackgroundColors[3]} 90deg 270deg,
      ${gaugeBackgroundColors[4]} 270deg 306deg,
      ${gaugeBackgroundColors[5]} 306deg 342deg,
      ${gaugeBackgroundColors[0]} 342deg 360deg
    )`;

  }, [halfGaugeId, gaugeBackgroundColors]);


  const createHand = useCallback(() => {
    const hand = getHand(percentage, handColor);
    const gaugeHandEl = document.getElementById(halfGaugeHandId);
    if (!gaugeHandEl || !hand || (!gaugeHandEl && !hand)) return;

    gaugeHandEl.style.background = hand;
  }, [halfGaugeHandId, percentage, handColor]);
  
  useEffect(() => {
    createStage();
    createHand();
  }, [percentage, createStage, createHand]);

  
  //__RESPONSIVE_SIZE___________________________________________________________________________________

  const makeResponsive = useCallback(() => {
    if (!halfGaugeContainerId) return;

    const containerEl = document.getElementById(halfGaugeContainerId);
    
    if (containerEl) {
      
      const parentElClientRect = containerEl?.parentElement?.getBoundingClientRect();
  
      if (parentElClientRect) {

        const parentWidth = parentElClientRect?.width;
        const parentHeight = parentElClientRect?.height;

        let scale = 1;

        if (parentWidth < 88) {
          scale = 0.2;
        } else if (parentWidth < 218) {
          scale = 0.4;
        } else if (parentWidth < 346 && parentHeight < 49) {
          scale = 0.5;
        } else if (parentWidth < 278 && parentHeight < 143) {
          scale = 0.7;
        } else if (parentWidth < 278 && parentHeight < 250) {
          scale = 0.7;
        } else if (parentWidth < 346 && parentHeight < 143) {
          scale = 0.7;
        } else if (parentWidth < 475 && parentHeight < 49) {
          scale = 0.5;
        } else if (parentWidth < 348) {
          scale = 0.9;
        }

        containerEl.style.transform = `scale(${scale})`;
      }
    }

    setTimeout(() => makeResponsive(), 500);
  }, [halfGaugeContainerId]);
  useEffect(() => responsive && makeResponsive(), [responsive, makeResponsive]);


  const displayContainer = useCallback(() => {
    const containerEl = document.getElementById(halfGaugeContainerId);
    if (containerEl) containerEl.style.opacity = 1;
  }, [halfGaugeContainerId]);
  useEffect(() => responsive && setTimeout(() => displayContainer(), 500), [responsive, displayContainer]);

  //____________________________________________________________________________________________________
  
  
  return <div id={halfGaugeContainerId} className={classes.container}>
    <div className={clsx(classes.minMax, classes.min)}>
      <p>
        {min}
      </p>
    </div>
    <div id={halfGaugeId} className={classes.gauge}>
      <div id={halfGaugeHandId} className={classes.gauge} />
      <div className={classes.valueDiv}>
        {percentageText && <p>{percentageText}</p>}
        {label && <p>{label}</p>}
      </div>
    </div>
    <div className={classes.minMax}>
      <p>
        {max}
      </p>
    </div>
  </div>;
};

export default HalfGauge;