

import { useUpdateEffect } from '../../utils';
import { Box, makeStyles, OutlinedTextFieldProps, TextField } from '@material-ui/core';
import React, { useState, useEffect, useRef } from 'react';

// keyCode constants
const BACKSPACE = 8;
const LEFT_ARROW = 37;
const RIGHT_ARROW = 39;
const DELETE = 46;
const SPACEBAR = 32;
const ENTER = 13;

export interface Props extends Omit<OutlinedTextFieldProps, 'onChange' | 'variant'> {
  numInputs?: number,
  onChange?: (otp: string) => void,
  separator?: React.ReactNode,
  inputStyle?: Object,
  value?: string,
  onConfirm?: (otp: string) => void
}

export interface SingleProps extends Partial<Omit<Props, 'onChange'>> {
  isLastChild: boolean,
  focus: boolean,
  index: number,
  onChange: (e: React.ChangeEvent<HTMLInputElement>) => void
}

const useStyles = makeStyles(theme => ({
  field: {
    '& [class*="MuiInputBase-root"]': {
      height: '100%',
      fontSize: 'inherit'
    },
    [theme.breakpoints.up('xs')]: {
      margin: theme.spacing(0),
      width: '1em', 
      height: '1em',
      fontSize: '.6em'
    },
    [theme.breakpoints.up('sm')]: {
      margin: theme.spacing(1),
      width: '1em', 
      height: '1em',
      fontSize: '.6em'
    },
    [theme.breakpoints.up('md')]: {
      margin: theme.spacing(1),
      width: '1.2em', 
      height: '1.2em',
      fontSize: '.8em'
    },
    [theme.breakpoints.up('lg')]: {
      margin: theme.spacing(2),
      width: '1.5em', 
      height: '1.5em', 
    },
    [theme.breakpoints.up('xl')]: {
      margin: theme.spacing(2),
      width: '1.6em', 
      height: '1.6em', 
    },
  },
  input: {
    textAlign: 'center',
    textTransform: 'uppercase',
    padding: 0
  }  
}));

const SingleOtpInput = ({
  separator,
  isLastChild,
  focus,
  autoFocus,
  index,
  value,
  placeholder,
  ...props
}: SingleProps) => {
  const classes = useStyles();
  const inputRef = useRef<any>();

  // Focus on first render
  // Only when autoFocus is true
  useEffect(() => {
    if (inputRef?.current && focus && autoFocus) inputRef.current.focus();
  }, [focus, autoFocus]);

  useUpdateEffect(() => {
    // Check if focusedInput changed
    // Prevent calling function if input already in focus
    if (inputRef?.current && focus) {
      inputRef.current.focus();
      inputRef.current.select();
    }
  }, [focus]);

  return <>
    <TextField
      className={classes.field}
      variant='outlined'
      inputProps={{
        ...props.inputProps,
        className: classes.input,
        autoComplete: 'off',
        maxLength: '1',
        ref: inputRef,
      }}
      value={value || ''}
      placeholder={placeholder && placeholder[index]}
      {...props}
    />
    {!isLastChild && separator}
  </>;
};

const OtpInput = ({
  numInputs = 6,
  onChange = (otp: string) => void(otp),
  value = '',
  onConfirm = (otp: string) => void(otp),
  ...props
}: Props) => {
  const [activeInput, setActiveInput] = useState(0);

  const [localValue, setLocalValue] = useState(value?.toString().split('') || []);
  const otpValue = value?.toString().split('') || localValue;

  // Helper to return OTP from input
  const handleOtpChange = (otp: string[]) => {
    setLocalValue(otp);
    onChange(otp.join('')?.toUpperCase());
  };
  
  // Focus on input by index
  const focusInput = (input: number) => setActiveInput(Math.max(Math.min(numInputs - 1, input), 0));
  
  // Focus on next input
  const focusNextInput = () => focusInput(activeInput + 1);
  
  // Focus on previous input
  const focusPrevInput = () => focusInput(activeInput - 1);
  
  // Change OTP value at focused input
  const changeCodeAtFocus = (value: string) => {
    const otp = otpValue;
    otp[activeInput] = value[0];

    handleOtpChange(otp);
  };

  // Handle pasted OTP
  const handleOnPaste = (e: React.ClipboardEvent) => {
    e.preventDefault();

    const { disabled } = props;

    if (disabled) return;
    
    const otp = otpValue;
    let nextActiveInput = activeInput;

    // Get pastedData in an array of max size (num of inputs - current position)
    const pastedData = e.clipboardData
      .getData('text/plain')
      .slice(0, numInputs - activeInput)
      .split('');

    // Paste data from focused input onwards
    for (let pos = 0; pos < numInputs; ++pos) {
      if (pos >= activeInput && pastedData.length > 0) {
        otp[pos] = pastedData.shift() || '';
        nextActiveInput++;
      }
    }

    setActiveInput(nextActiveInput);

    setTimeout(() => {
      focusInput(nextActiveInput);
      handleOtpChange(otp);
    }, 0);
  };

  const handleOnChange = (e: React.ChangeEvent<HTMLInputElement>) => changeCodeAtFocus(e.target.value);    
  
  // Handle cases of backspace, delete, left arrow, right arrow, space
  const handleOnKeyDown = (e: React.KeyboardEvent<HTMLInputElement>) => {
    if (e.keyCode === BACKSPACE || e.key === 'Backspace') {
      e.preventDefault();
      changeCodeAtFocus('');
      focusPrevInput();
    } else if (e.keyCode === DELETE || e.key === 'Delete') {
      e.preventDefault();
      changeCodeAtFocus('');
    } else if (e.keyCode === LEFT_ARROW || e.key === 'ArrowLeft') {
      e.preventDefault();
      focusPrevInput();
    } else if (e.keyCode === RIGHT_ARROW || e.key === 'ArrowRight') {
      e.preventDefault();
      focusNextInput();
    } else if (e.keyCode === SPACEBAR || e.key === ' ' || e.key === 'Spacebar' || e.key === 'Space') {
      e.preventDefault();
    } else if (e.keyCode === ENTER || e.key === 'Enter') {
      onConfirm(otpValue.join('')?.toUpperCase());
    }
  };

  // The content may not have changed, but some input took place hence change the focus
  const handleOnInput = () => focusNextInput();

  const handleFocus = (i: number) => (e: React.FocusEvent<HTMLInputElement | HTMLTextAreaElement>) => {
    setActiveInput(i);
    e.target.select();
  };

  const handleBlur = () => setActiveInput(-1);

  return (
    <Box display='flex' alignItems='center' justifyContent='center' fontSize='8em'>
      {Array(numInputs).fill(0).map((_, i) => (
        <SingleOtpInput
          {...props}
          key={i}
          index={i}
          focus={activeInput === i}
          value={otpValue[i]}
          onChange={handleOnChange}
          onKeyDown={handleOnKeyDown}
          onInput={handleOnInput}
          onPaste={handleOnPaste}
          onFocus={handleFocus(i)}
          onBlur={handleBlur}
          isLastChild={i === numInputs - 1}
        />
      ))}
    </Box>
  );
};

export default OtpInput;