import { Box, Button, DialogActions, Divider, ListItem, ListItemIcon, ListItemText, MobileStepper, Paper, Step, StepButton, StepConnector, StepLabel, Stepper as MuiStepper, Typography, useMediaQuery, useTheme, withStyles } from '@material-ui/core';
import clsx from 'clsx';
import React, { useEffect, useState } from 'react';
import { IconContext } from 'react-icons/lib';
import { MdCheck, MdChevronLeft, MdChevronRight } from 'react-icons/md';

const ColorlibConnector = withStyles(theme => ({
  alternativeLabel: ({ orientation }) => orientation === 'vertical' ? {
    top: -28,
    transform: 'rotate(90deg)',
    left: 'calc(-50% + 40px)',
    right: 'calc(50% + 10px)',
    '&>span': {
      minHeight: 3
    }
  } : {
    top: 22,
  },
  active: {
    '& $line': {
      backgroundPosition: 'left bottom'
    },
  },
  completed: {
    '& $line': {
      backgroundPosition: 'left bottom',
      backgroundImage: `linear-gradient( to right, ${theme.palette.secondary.main} 0%,${theme.palette.secondary.main} 50%,#eaeaf0 51%, #eaeaf0 100%)`,
    },
  },
  line: {
    height: 3,
    border: 0,
    backgroundImage: `linear-gradient( to right, ${theme.palette.secondary.main} 0%,${theme.palette.primary.main} 50%,#eaeaf0 51%, #eaeaf0 100%)`,
    backgroundSize: '200% 100%',
    backgroundPosition: 'right bottom',
    transition: 'background-position 500ms ease',
    borderRadius: 1,
  }
}))(StepConnector);

const ColorlibStepIcon = withStyles(theme => ({
  root: {
    zIndex: 1,
    color: '#fff',
    width: 50,
    height: 50,
    display: 'flex',
    borderRadius: '50%',
    justifyContent: 'center',
    alignItems: 'center',
    fontSize: 24,
    backgroundColor: theme.palette.type === 'light' ? '#ccc' : '#838383',
    transition: 'all 400ms ease 100ms'
  },
  initial: theme.palette.type === 'light' ? {
    backgroundColor: ({ ambienteTeste }) => ambienteTeste ? '#ffc7a2' : '#9acbb8'
  } : {
    backgroundColor: ({ ambienteTeste }) => ambienteTeste ? '#8b6751' : '#507265'
  },
  active: {
    backgroundColor: ({ ambienteTeste }) => ambienteTeste ? '#f5781e !important' : '#00995D !important',
    boxShadow: theme.shadows[5],
  },
  completed: {
    backgroundColor: '#2979ff !important',
  },
  error: {
    backgroundColor: '#f44336 !important',
  },
  icon: {
    '& path': {
      stroke: '#FFF'
    }
  }
}))(({ initial, active, completed, icon, classes, error }) => (
  <div
    className={clsx(classes.root, {
      [classes.initial]: initial,
      [classes.active]: active,
      [classes.completed]: completed,
      [classes.error]: error,
    })}
  >
    <IconContext.Provider value={{ className: classes.icon }} >
      {icon}
    </IconContext.Provider>
  </div>
));

const stepperStyles = theme => ({
  root: {
    '&>div': {
      minHeight: 'calc(100vh - 50px)'
    }
  },
  header: {
    background: theme.palette.background.default,
    marginTop: -10,
  },
  caption: {
    display: 'block',
    textAlign: 'center',
    color: theme.palette.text.hint
  },
  selectedCaption: {
    color: theme.palette.text.secondary
  },
  stepper: {
    padding: ({ orientation }) => orientation === 'vertical' ? '0 0 0 136px' : '',
    [theme.breakpoints.down('sm')]: {
      padding: 0
    },
    '&>div': {
      display: 'flex',
      padding: ({ orientation }) => orientation === 'vertical' ? '0 0 30px' : '',
      '&>button': {
        '&>span:nth-child(1)': {
          flexDirection: ({ orientation }) => orientation === 'vertical' ? 'row' : 'column'
        }
      }
    }
  },
  subDescription: {
    minWidth: 140,
    display: 'flex',
    alignItems: 'center',
    justifyContent: 'end',
    position: 'absolute',
    top: 17,
    left: -148,
  },
  icon: {
    fontSize: theme.typography.fontSize * 2,
    '& path': {
      stroke: theme.palette.text.primary
    }
  },
  mobileDivider: {
    marginBottom: theme.spacing(2)
  },
  stepLabel: {
    '&>span:nth-child(2)': {
      margin: ({ orientation }) => orientation === 'vertical' ? '0 0 0 8px' : '',
      '&>span:nth-child(1)': {
        margin: ({ orientation }) => orientation === 'vertical' ? 0 : ''
      }
    },
  }
});

const Stepper = ({ 
  children, 
  classes = {}, 
  steps = [], 
  onClick = (index) => (event) => void(index, event), 
  onNext = (event) => void(event), 
  onPrevious = (event) => void(event), 
  onFinish = (event) => void(event), 
  maxStep: maxStepProps, 
  activeStep = 0,
  errors = [],
  component: Component = 'div',
  backLabel = 'Voltar',
  nextLabel = 'Avançar',
  finishLabel = 'Finalizar',
  disableFinish = false,
  disableNext = false,
  disablePrevious = false,
  showProgress = false,
  disableActions = false,
  orientation = 'horizontal',
  ...props
}) => {
  const theme = useTheme();
  const maxSteps = steps.length;
  const maxStep = maxStepProps || steps.length - 1;
  const mobile = useMediaQuery(theme.breakpoints.down(Math.max(maxSteps * 100, theme.breakpoints.values.sm)));
  const ambienteTeste = JSON.parse(window.localStorage.getItem('ambienteTeste'));
  const [initialStep, setInitialStep] = useState(-1);
  useEffect(() => (activeStep > initialStep && showProgress) && setInitialStep(activeStep), [activeStep, initialStep, showProgress]);

  return <Component {...props}>
    {mobile && orientation !== 'vertical' ? (
      <Paper square elevation={0}>
        <ListItem dense className={classes.header}>
          <ListItemIcon className={classes.icon}>{steps[activeStep][0]}</ListItemIcon>
          <ListItemText primary={steps[activeStep][1]} secondary={steps[activeStep][2]}/>
        </ListItem>
        <Divider className={classes.mobileDivider}/>
      </Paper>
    ) : (
      <MuiStepper 
        className={classes.stepper} 
        alternativeLabel 
        activeStep={activeStep} 
        orientation={orientation}
        connector={<ColorlibConnector />}
      >
        {steps.map(([icon, label, description, subDescription], i) => (
          <Step key={label}>
            {orientation === 'vertical' && subDescription &&
              <div className={classes.subDescription}>
                <Typography 
                  variant='caption' 
                  className={clsx(classes.caption, { [classes.selectedCaption]: activeStep >= i })}>
                  {subDescription}
                </Typography>
              </div>
            }
            <StepButton 
              onClick={onClick(i)}
              disabled={i > maxStep} 
              optional={
                <>
                  <Typography 
                    variant='caption' 
                    className={clsx(classes.caption, { [classes.selectedCaption]: activeStep >= i })}>
                    {description}
                  </Typography>
                  {orientation !== 'vertical' && subDescription && 
                    <Typography 
                      variant='caption'
                      className={clsx(classes.caption, { [classes.selectedCaption]: activeStep >= i })}>
                      {subDescription}
                    </Typography>
                  }
                </>
              }
            >
              <StepLabel 
                className={classes.stepLabel}
                disabled={i > maxStep} 
                error={errors[i]} 
                StepIconComponent={ColorlibStepIcon}
                StepIconProps={{ icon, error: errors[i], initial: initialStep >= i, ambienteTeste }}
              >
                {label}
              </StepLabel>
            </StepButton>
          </Step>
        ))}
      </MuiStepper>
    )}

    {children && 
      <Box padding={mobile ? 0 : 4} pt={1} pb={0}>
        {children}
      </Box>
    }

    {mobile && orientation !== 'vertical' ? (
      <MobileStepper
        steps={maxSteps}
        position='bottom'
        variant='dots'
        activeStep={activeStep}
        backButton={
          <Button 
            startIcon={<MdChevronLeft/>} 
            onClick={onPrevious} 
            disabled={activeStep === 0}>
            {backLabel}
          </Button>
        }
        nextButton={
          <Button 
            variant='contained' 
            color='primary' 
            endIcon={activeStep === maxSteps - 1 ? <MdCheck/> : <MdChevronRight/>} 
            onClick={activeStep === steps.length - 1 ? onFinish : onNext}>
            {activeStep === steps.length - 1 ? finishLabel : nextLabel}            
          </Button>
        }
      />
    ) : !disableActions ? (
      <DialogActions>
        <Box padding={mobile ? 0 : 4} paddingTop={1} paddingBottom={1}>
          <Button 
            disabled={activeStep === 0 || disablePrevious} 
            onClick={onPrevious}>
            {backLabel}
          </Button>
          <Button
            variant='contained'
            color='primary'
            onClick={activeStep === steps.length - 1 ? onFinish : onNext}
            disabled={activeStep === steps.length - 1 ? disableFinish : disableNext}
          >
            {activeStep === steps.length - 1 ? finishLabel : nextLabel}
          </Button>
        </Box>
      </DialogActions>
    ) : null }
  </Component>;
};

export default withStyles(stepperStyles, { name: 'ArgousStepper' })(Stepper);