import { serverAction } from './api';
import { getStore } from './store';
import { getFileFromLink, headers } from '../api';
import { ModalProviderContext } from '../components/ConfirmModal/ModalProvider';
import { setAnotherProgramField, setField } from '../fieldsRedux';
import { ActionType, ArgousType } from '../types';
import { useTheme } from '@material-ui/core';
import * as colors from '@material-ui/core/colors';
import { ChartOptions } from 'chart.js';
import DomToImage from 'dom-to-image';
import { BasicModalProps } from 'lib/components';
import { isNumber } from 'lodash';
import moment from 'moment';
import React, { useRef, useEffect, useContext, useState } from 'react';


export const widthToCenterVertically = (content:number) => {
  const B = document.body;
  const H = document.documentElement;
  const height = Math.max(B.scrollHeight, B.offsetHeight, H.clientHeight, H.scrollHeight, H.offsetHeight);
  return (height - content) / 2;
};

export const getCookie = (cName: string) => {
  if (document.cookie.length > 0) {
    let cStart = document.cookie.indexOf(cName + '=');
    if (cStart !== -1) {
      cStart = cStart + cName.length + 1;
      let cEnd = document.cookie.indexOf(';', cStart);
      if (cEnd === -1) {
        cEnd = document.cookie.length;
      }
      return unescape(document.cookie.substring(cStart, cEnd));
    }
  }
  return '';
};

export const addExtraZero = (vl: number|string) => (vl < 10) ? `0${typeof vl === 'string' ? parseInt(vl) : vl}` : vl;

export const formataNome = (value:string) => {
  if (value) {
    let nome = value.toLowerCase().split(' ');
    for (let i = 0; i < nome.length; i++)
      nome[i] = nome[i].charAt(0).toUpperCase() + nome[i].substring(1);
    return nome.join(' ');
  }
  return '';
};

export function downloadBase64(base64: string, fileName: string, fileExtension: string) {
  let element = document.createElement('a');
  element.setAttribute('href', base64);
  element.setAttribute('download', fileExtension ? `${fileName}.${fileExtension}` : fileName);
  element.style.display = 'none';
  document.body.appendChild(element);
  element.click();
  document.body.removeChild(element);
}

/**
 * Função faz o download de qualquer url
 * 
 * @param {string} url - Url a ser baixada
 * @param {string} fileName - Nome do arquivo baixado
 * @param {string=} fileExtension - Extenção do arquivo baixado
 * 
 * @returns {void}
 */
export async function downloadFromUrl(url: string, fileName: string, fileExtension?: string): Promise<void> {
  const blob = await (await fetch(url, { method: 'GET', headers: headers() })).blob();
  const urlCreator = URL || window.URL || window.webkitURL; 
  const tag = document.createElement('a');
  tag.href = urlCreator.createObjectURL(blob);
  tag.download = fileExtension ? `${fileName}.${fileExtension}` : fileName;
  tag.style.display = 'none';
  document.body.appendChild(tag);
  tag.click();
  document.body.removeChild(tag);
}

export const printPdf = async (URL = '') => {
  const url = await getFileFromLink({ URL });

  const iframe = document.createElement('iframe');
  document.body.appendChild(iframe);
  
  iframe.style.display = 'none';
  iframe.onload = () => iframe.contentWindow?.print();
  
  if (iframe.contentWindow) iframe.contentWindow.onafterprint = () => document.removeChild(iframe);
  
  iframe.src = url;
};

export const getRandomColor = () => {
  let letters = '0123456789ABCDEF';
  let color = '#';
  for (let i = 0; i < 6; i++)
    color += letters[Math.floor(Math.random() * 16)];
  return color;
};

interface RNG {
  state:any;
  m:number;
  a:number;
  c:number;
}

class RNG { //pseudo random
  constructor(seed:number) {
  // LCG using GCC's constants
    this.m = 0x80000000; // 2**31;
    this.a = 1103515245;
    this.c = 12345;

    this.state = seed ? seed : Math.floor(Math.random() * (this.m - 1));
  }
  nextInt() {
    this.state = (this.a * this.state + this.c) % this.m;
    return this.state;
  }
  nextFloat() {
    return this.nextInt() / (this.m - 1);
  }
  nextRange(start: number, end: number) {
    var rangeSize = end - start;
    var randomUnder1 = this.nextInt() / this.m;
    return start + Math.floor(randomUnder1 * rangeSize);
  }
  choice(array:any) {
    return array[this.nextRange(0, array.length)];
  }
}

export const getInitials = (nome = '') => `${nome.split(' ')[0].substring(0, 1).toUpperCase()}${nome.split(' ')[nome.split(' ').length - 1].substring(0, 1).toUpperCase()}`;

export const getNumeroFromInitials = (initials: string, length: number) => Math.ceil(new RNG((initials.charCodeAt(0) + initials.charCodeAt(1) - 130) * length / 5).nextFloat() * 10) - 1;

export const usePrevious = (value:any) => {
  const ref = useRef();
  useEffect(() => {
    ref.current = value; 
  });
  return ref.current;
};

/**
 * Igual o useEffect, mas não ativa quando o componente monta
 * 
 * @param {React.EffectCallback} effect - Função imperativa que pode retornar uma função de limpeza
 * @param {React.DependencyList} dependencies - Dependencias que quando atualizadas rodarão o efeito
 * 
 * @returns {void} Sem retorno
 */
export function useUpdateEffect(effect: React.EffectCallback, dependencies: React.DependencyList): void {
  const isInitialMount = useRef(true);
  useEffect(() => {
    if (isInitialMount.current) isInitialMount.current = false;
    else effect();    
  }, dependencies);
}

export function formataDinheiro(n: number) {
  if (n >= 0) return new Intl.NumberFormat('pt-BR', { minimumFractionDigits: 2 }).format(n);
  return `(${new Intl.NumberFormat('pt-BR', { minimumFractionDigits: 2 }).format(Math.abs(n))})`;
}

export const sanitizaHTML = (html = '') => html?.replace(/(<script>)|(<\/script>)|(\bon[^ ]*=('|"))/gi, '') || '';


/**
 * Format bytes as human-readable text.
 * 
 * @param {number} bytes - Number of bytes.
 * @param {boolean} si - True to use metric (SI) units, aka powers of 1000. False to use binary (IEC) units, aka powers of 1024.
 * @param {number} dp - Number of decimal places to display.
 * 
 * @returns {string} Formatted string.
 */
export const humanFileSize = (bytes: number, si: boolean = true, dp: number = 2): string => {
  const thresh = si ? 1000 : 1024;
                                
  if (Math.abs(bytes) < thresh) return `${bytes} B`; 

  const units = si 
    ? ['kB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'] 
    : ['KiB', 'MiB', 'GiB', 'TiB', 'PiB', 'EiB', 'ZiB', 'YiB'];
  let u = -1;
  const r = 10 ** dp;

  do {
    bytes /= thresh;
    ++u;
  } while (Math.round(Math.abs(bytes) * r) / r >= thresh && u < units.length - 1);

  return `${bytes.toFixed(dp)} ${units[u]}`;
};

const chaves = Object.keys(colors).filter(i => i !== 'common');

export const randomColor = (): string => {
  const color = colors[chaves[Math.floor(chaves.length * Math.random())]];
  const chavesCor = Object.keys(color);
  return color[chavesCor[Math.floor(chavesCor.length * Math.random())]];
};

/**
 * Hook para padronizar o tema dos gráficos
 * 
 * @param {ChartOptions} props - Props customizados
 * @returns {ChartOptions} - retorno com tema ajustado
 */
export const useGraph = ({ 
  title, legend: { labels, ...legend } = {}, 
  plugins: { datalabels, ...plugins } = {}, 
  scales: { 
    yAxes: [{ ticks: yticks = {}, gridLines: ygridLines = {}, ...yAxes } = {}, ...y] = [], 
    xAxes: [{ ticks: xticks = {}, gridLines: xgridLines = {}, ...xAxes } = {}, ...x] = [], 
    ...scales 
  } = {}, 
  ...props 
}: ChartOptions = {}): ChartOptions => {
  const theme = useTheme();
  return { 
    title: { fontColor: theme.palette.text.primary, fontSize: theme.typography.fontSize, ...title },
    legend: { position: 'bottom', labels: { fontColor: theme.palette.text.primary, ...labels }, ...legend },
    plugins: { datalabels: {
      align: 'end', anchor: 'end',
      color: theme.palette.text.secondary,
      textShadowColor: theme.palette.background.default, textShadowBlur: 3, ...datalabels
    }, ...plugins }, 
    scales: {
      yAxes: [{         
        ticks: { fontColor: theme.palette.text.secondary, ...yticks }, 
        gridLines: { color: theme.palette.divider, zeroLineColor: theme.palette.divider, ...ygridLines },
        ...yAxes
      }, ...y],
      xAxes: [{ 
        ticks: { fontColor: theme.palette.text.secondary, ...xticks }, 
        gridLines: { color: theme.palette.divider, zeroLineColor: theme.palette.divider, ...xgridLines },
        ...xAxes
      }, ...x],
      ...scales
    }, ...props 
  };
};

export const useModal = () => {
  const createModal = useContext(ModalProviderContext);
  return createModal;
};

export const useStore = (): ArgousType => {
  return getStore().getState().argous || {};
};

export const reduxDispatch = (action: ActionType) => {
  getStore().dispatch(action);
};

export const formataTelefone = (value: string) => {
  if (!value) return value; 
  const currentValue = value.replace(/[^\d]/g, '');
  const cvLength = currentValue.length; 
  if (cvLength < 3) return currentValue; 
  if (cvLength < 7) return `(${currentValue.slice(0, 2)}) ${currentValue.slice(2)}`; 
  if (cvLength < 11) return `(${currentValue.slice(0, 2)}) ${currentValue.slice(2, 6)}-${currentValue.slice(6)}`; 
  return `(${currentValue.slice(0, 2)}) ${currentValue.slice(2, 7)}-${currentValue.slice(7, 11)}`; 
};

export const useWebSocket = () => {
  const [value, setValue] = useState({});

  useEffect(() => {
    setField('WSHandler', { onMessage: setValue });
  }, []);

  return value;
};

export const formataCpf = (value: string) => value?.replace(/[^\d]/, '').replace(/^(.{3})(.{3})(.{3})(.{2})$/, '$1.$2.$3-$4');

export const numbersOnly = (value: string, type: string) => value ? value.replace(type === 'float' ? /[^0-9/.,]/g : /[^0-9]/g, '') : '';

export const currencyMask = (value: string | number) => {
  if (value === 0) return `${value.toString()},00`;
  if (!value) return '';
  if (isNumber(value)) value = value.toFixed(2);
  value = value.toString();
  value = value.replace(/\D/g, '');
  value = value.replace(/(\d)(\d{2})$/, '$1,$2');
  value = value.replace(/(?=(\d{3})+(\D))\B/g, '.');
  return value;
};

export const toFloat = (value: string) => {
  if (!value) return;
  value = value.replace(/[.]/g, '');
  value = value.replace(/[,]/g, '.');
  return parseFloat(value);
};

export const copyToClipboard = (text: string) => {
  if (navigator.clipboard) navigator.clipboard.writeText(text);
  else {
    const textField = document.createElement('input');
    textField.setAttribute('value', text);
    document.body.appendChild(textField);
    textField.focus();
    textField.select();
    textField.setSelectionRange(0, 99999);
    document.execCommand('copy');
    textField.remove();
  }
};

export const confirm = (props: BasicModalProps | string) => new Promise<boolean>((resolve, reject) => {
  setAnotherProgramField('openModal', { value: Object.assign({ resolve, reject, okLabel: 'Confirmar', cancelLabel: 'Cancelar', title: 'Confirmar?' }, typeof props === 'string' ? { description: props } : props) }, 'argousSystem');
});

export const storageMeasures = {
  byte: 'B',
  kilobyte: 'KB',
  megabyte: 'MB',
  gigabyte: 'GB',
  terabyte: 'TB',
  petabyte: 'PB',
  exabyte: 'EB'
};

export const bytesTranslator = (bytes: number = 0, decimals: number = 2, binary: boolean = false): object => {
  const measures = Object.values(storageMeasures);
  const divisor = ((binary) ? 1024 : 1000);
  decimals = ((decimals < 0) ? 0 : decimals);
  let index = Math.floor(Math.log(bytes) / Math.log(divisor));
  if (index < 0) index = 0;
  const size = parseFloat((bytes / Math.pow(divisor, index)).toFixed(decimals));
  const type = measures[index];
  return { type, size };
};

export const formatCpfCnpj = (value: (string|number)) => {
  if (!value) return '';
  if (isNumber(value)) value = value.toString();
  value = value.replace(/[^\d]/g, '');
  const valueLen = value.length;
  if (valueLen > 3 && valueLen < 7) value = `${value.substring(0, 3)}.${value.substring(3)}`;
  else if (valueLen > 6 && valueLen < 10) value = `${value.substring(0, 3)}.${value.substring(3, 6)}.${value.substring(6)}`;
  else if (valueLen > 8 && valueLen < 12) value = `${value.substring(0, 3)}.${value.substring(3, 6)}.${value.substring(6, 9)}-${value.substring(9)}`;
  else if (valueLen > 11 && valueLen < 13) value = `${value.substring(0, 2)}.${value.substring(2, 5)}.${value.substring(5, 8)}/${value.substring(8)}`;
  else if (valueLen > 12) value = `${value.substring(0, 2)}.${value.substring(2, 5)}.${value.substring(5, 8)}/${value.substring(8, 12)}-${value.substring(12, 14)}`;
  return value;
};

export function dataUrlToBlob(dataUrl: string) {
  if (!dataUrl) return;
  let byteString = atob(dataUrl.split(',')[1]);
  let mimeString = dataUrl.split(',')[0].split(':')[1].split(';')[0];
  let arrayBuffer = new ArrayBuffer(byteString.length);
  let intArray = new Uint8Array(arrayBuffer);
  for (let i = 0; i < byteString.length; i++) {
    intArray[i] = byteString.charCodeAt(i);
  }
  let blob = new Blob([arrayBuffer], { type: mimeString });
  return blob;
}

export function cpfTest(cpf: (string|number)) {

  if (cpf === '00000000000' || !cpf) return false;
  if (typeof cpf === 'number') cpf = cpf.toString();

  let sum = 0;
  let rest: number;

  for (let index = 1; index <= 9; index++) sum += (parseInt(cpf.substring(index - 1, index)) * (11 - index));
  rest = ((sum * 10) % 11);

  if ((rest === 10) || (rest === 11)) rest = 0;
  if (rest !== parseInt(cpf.substring(9, 10))) return false;

  sum = 0;
  for (let index = 1; index <= 10; index++) sum += (parseInt(cpf.substring(index - 1, index)) * (12 - index));
  rest = ((sum * 10) % 11);

  if ((rest === 10) || (rest === 11)) rest = 0;
  if (rest !== parseInt(cpf.substring(10, 11))) return false;

  return true;
}

interface PrintScreenParameters { 
  /**
   * Event
   */
  event?: Event, 
  /**
   * Element id
   */
  id: string, 
  /**
   * Auto download
   */
  download: boolean 
}
export const printScreen = async ({ event, id = 'root', download = false }: PrintScreenParameters): Promise<object> => {
  let status = false;
  let file;
  let linkElement;

  const element = document.getElementById(id);
  if (element) {

    const dataUrl = await DomToImage.toPng(element);
    if (dataUrl) {
      linkElement = document.createElement('a');
      linkElement.download = `print_screen_${moment(new Date()).format('DD-MM-YYYY_HH:mm:ss')}.png`;
      linkElement.href = dataUrl;

      const blob = dataUrlToBlob(dataUrl);
      if (blob) file = new File([blob], linkElement.download, { type: blob.type });
    }
  }

  (linkElement && file) && (status = true);
  (linkElement && download) && linkElement.click();

  return { event, status, file, element, link: linkElement };
};

interface GeraLogInterface { 
  (
    /**
     * Id do documento
     */
    idDocumento: number, 
    /**
     * Descrição
     */
    descricao: string, 
    /**
     * Tipo de ação
     * 
     * 1 - Upload
     * 2 - Alteração
     * 3 - Consulta
     */
    tpAcao: string, 
    /**
     * Token do sistema
     */
    tokenSistema?: string
  ): Promise<void> 
}
export const geraLog: GeraLogInterface = async (idDocumento, descricao, tpAcao, tokenSistema) => {
  if (!getStore()?.getState()?.argous?.usuarioLogado?.idUsuario) return;
  await serverAction('geraLog', { idDocumento, descricao, tpAcao }, 'CtrlDocumentos', false, tokenSistema);
};

export const printUrl = async (url: string): Promise<void> => {
  const res = await fetch(url);
  const file = await res.blob();
  let iframe = document.createElement('iframe');
  document.body.appendChild(iframe);
  iframe.style.display = 'none';
  iframe.src = window.URL.createObjectURL(file);
  iframe.onload = () => {
    setTimeout(() => {
      iframe.focus();
      iframe?.contentWindow?.print();
    }, 1);
  };
};

declare global {
  interface Window {
    clipboardData: any;
  }
}
export function stopPrntScr(e: KeyboardEvent) {
  if (e.key === 'PrintScreen') {
    e.preventDefault();
    e.stopPropagation();
    try {
      window.clipboardData.setData('text', 'Access Restricted');
    } catch (err) {}
  }
  let inpFld = document.createElement('input');
  inpFld.setAttribute('value', '.');
  inpFld.setAttribute('width', '0');
  inpFld.style.height = '0px';
  inpFld.style.width = '0px';
  inpFld.style.border = '0px';
  document.body.appendChild(inpFld);
  inpFld.select();
  document.execCommand('copy');
  inpFld.remove();
  return false;
}